import agentdojo.default_suites.v1_1_2.travel.injection_tasks
import agentdojo.default_suites.v1_1_2.workspace.injection_tasks
import agentdojo.default_suites.v1_1_2.workspace.user_tasks  # noqa: F401
